<?php
/**
 * @package Keyinvoice
 */

namespace KeyinvoiceWordpress\API;
use KeyinvoiceWordpress\API\Keyinvoice;
use KeyinvoiceWordpress\API\Wp;

class Ajax 
{
    public $actions = [];
	public $wp;

    public function register()
    {
		$this->wp = new Wp();
        $this->actions = [
            'test_api' => 'testAjax',
			'import_products' => 'importProducts',
			'generate_document' => 'generateDocument'

        ];

        foreach( $this->actions as $action => $callback )
        {  
            if(  method_exists( $this, $callback ) )
                add_action( 'wp_ajax_'.$action,[ $this, $callback ] );
        }
    }

	public function generateDocument()
    {
    	$post = $_POST;
    	$soap = new Keyinvoice( get_option( 'url_api') );
		$soap->authenticate( get_option( 'api_key') );
    	$doc = $soap->generateInvoice( $post['id'] );
		
		echo json_encode(array( 'msg' => $doc));
		wp_die(); 

    }

    public function testAjax()
    {
        $post = $_POST;
		$api_url = sanitize_text_field($post['api_url']);
		$api_key = sanitize_text_field($post['api_key']);

		if( $api_url != "" && $api_key != "" )
		{
			try
			{
				$soap = new Keyinvoice( $api_url );
				$result = $soap->authenticate( $api_key );
				if( intval($result[0]) != 1 )
				{
					echo json_encode(array( 'msg' => __('Autenticação falhada. Verifique a configuração da sua ligação API!')));
				}else
				{
					$company = $soap->company( $result[1] );
					$msg = __('Autenticação feita com sucesso!').'</br>';
					$msg .= __("Empresa") . ":  " .  utf8_decode($company->DAT[0]->Name);
					echo json_encode(array('msg' => $msg ));	
				}
            } catch (Exception $ex) 
			{
			   echo json_encode(array( 'msg' => __('Autenticação falhada. Verifique a configuração da sua ligação API!'), 'erroe' => $ex));	
			}
		}	
		wp_die(); 
    }

	public function importProducts()
	{
		$soap = new Keyinvoice( get_option( 'url_api') );
		$soap->authenticate( get_option( 'api_key') );
		
		$result = $soap->countProducts();
		$total = $result[1];
		if( $result[0] == '-600' )
		{
			echo "<br/> " . $soap->responseMessage( $result[0], 0 );
			die;
		}
		$numProducts = intval( $result[1] );
		for( $i = 0; $i < $numProducts; $i += 25 )
		{
			$result = $soap->listProducts( $i );
			if( intval($result->RC) != 1 )
			{
				if( $result->RC == '-600' )
				{
					echo "<br/> " . $this->mydecode( $soap->responseMessage( $result->RC, 0 ) );
					return;
				}

				echo "<br/>" . __('Erro ao ler lista de produtos.');
				return;
			}
			
			for( $j=0; $j < count($result->DAT); ++$j )
			{
				$product = $result->DAT[$j];

				$price = floatval($product->Price);
				if( intval($product->TaxIncluded) == 1 )
					$price = floatval($product->Price) / (1+ floatval($product->TAX)/100);

				$tax_class_name = 'Ki IVA '.$product->TAX;
				$tax_class_slug = 'ki-iva-'.$product->TAX;

				$tax_rate = $product->TAX;
				$tax_rate_name = 'Taxa '.$product->TAX;	

				$existing       = \WC_Tax::get_tax_classes();
				if ( ! in_array( $tax_class_name, $existing, true ) ) 
				{
					\WC_Tax::create_tax_class($tax_class_name,$tax_class_slug);
					\WC_Cache_Helper::invalidate_cache_group( 'taxes' );
					\WC_Cache_Helper::get_transient_version( 'shipping', true );

					$tax_rate_data = array(
						'tax_rate_country' => '*',
						'tax_rate_state' => '*',
						'tax_rate' => $tax_rate,
						'tax_rate_name' => $tax_rate_name,
						'tax_rate_priority' => 1,
						'tax_rate_compound' => 0,
						'tax_rate_shipping' => 1,
						'tax_rate_order' => 0,
						'tax_rate_class' => $tax_class_slug
					);
					\WC_Tax::_insert_tax_rate( $tax_rate_data );
				}
			
				$family_tree=array();
				if(isset($product->FamilyTree) && count($product->FamilyTree)>0){
					foreach($product->FamilyTree as $fam){
						$family_tree[]=$fam->FamilyName;
					}
				}
				if(($weight=$soap->getProductInfo($product->Ref,"weight"))===false) $weight='';
				$this->wp->setProduct([
					'type'               => '',
					'name'               => $product->Name,
					'description'        => $product->LongDescription,
					'short_description'  => $product->ShortDescription,
					'sku'                => $product->Ref,
					'regular_price'      => $price, 
					'reviews_allowed'    => true,
					'virtual' 			 => false,
					'manage_stock'		 => (($product->HasStocks==1)?true:false),
					'stock_qty'			 => $product->Stock,
					'image_url' 		 => $product->Image,
					'family'			 => $product->Family,
					'family_tree'		 => $family_tree,
					'tax_class' 		 => $tax_class_name,
					'woocommerce_calc_taxes' => 'yes',
					'weight'			 => $weight,
					'brand'				 => $product->Brand->BrandName
				], $product->Ref );
			}
		}
		echo json_encode(array( 'msg' => __('Importaçao Finalizada com sucesso! '.$total.' Artigo(s) sincronizado(s)')));die;
	}

    
}
